--[[ 
A grid that always shows the shortest path from the center of the grid to the mouse.

Current problems : 

It can only find paths in the upper-left quadrant for some reason. I cannot get it to find a full path in any other quadrant. 

Downsides of the simulation : I cant go step by step to prevent crashing, nor see the exact place it decides to loop out. 

For some reason its making the wrong axis preference choice. 

---
How its supposed to work, 

Ai starts in the middle of the grid, 

examines which moves around itself are possible (out of up down left right, no diagonal)
eliminates any of those possible moves that make it farther from the target 
it then finds which axis is farthest, if equal it picks a random one 
Then it matches the axis preference to the possible moves in the table, and adds that move to the path 
Breaking its own loop when the path has reached an appropriate distance from the target. 

So if there aren't any obvious problems, Im just gonna start over with a much more debuggable system that I can easily throw and rework ideas at disregarding any form of speed. 
The code here has already gotten allot slower trying to bandaid or debug haha. 
]]
local fps = require("fps")
local grid = {} 
for x = 1,48 do 
	grid[x] = {} 
	for y = 1,48 do 
		grid[x][y] = {}
	end
end

local mouseover = nil -- either nil or the cell the mouse is currently over. 

local current_path = {} -- the current path that will be displayed in the game 
local path_timer = 0 
local path_timermax = 1/3

function absdistance( x1, y1, x2, y2 )
	return math.sqrt( (x2-x1)^2 + (y2-y1)^2 )
end
function single_distance(a1,a2)
	return math.abs(a1-a2)
end
function update_mouseover()
	local mouse_x,mouse_y = love.mouse:getPosition()
	local new_mouseover = {} 
	new_mouseover.x = math.floor(mouse_x/15)
	new_mouseover.y = math.floor(mouse_y/15)
	mouseover = new_mouseover
end

function update_path(x1,y1,x2,y2)
	--create path table and set first cell to 24x 24y(center)
	local path = {} 
	local first_cell = {} 
	first_cell.x = 24 
	first_cell.y = 24
	table.insert(path,first_cell)
	local pathcomplete = false  
	print("Made path table")
	while (pathcomplete ~= true) do 
		local possible_positions = {}
		local curx = path[#path].x
		local cury = path[#path].y
		--insert up left right down if in bounds
		if cury - 1 >= 1 and cury - 1 <= 48 then -- up 
			local up = {} 
			up.x = curx 
			up.y = cury - 1 
			up.type = "up"
			table.insert(possible_positions,up)
		end
		if cury + 1 >= 1 and cury + 1 <= 48 then -- down
			local down = {} 
			down.x = curx 
			down.y = cury + 1 
			down.type = "down"
			table.insert(possible_positions,down)
		end
		if curx - 1 >= 1 and curx - 1 <= 48 then -- left 
			local left = {} 
			left.x = curx - 1 
			left.y = cury 
			left.type = "left"
			table.insert(possible_positions,left)
		end
		if curx + 1 >= 1 and curx + 1 <= 48 then -- right
			local right = {} 
			right.x = curx + 1 
			right.y = cury 
			right.type = "right"
			table.insert(possible_positions,right)
		end
		--
		--eliminate moves that make us farther
		if #possible_positions >= 1 then  
			print("More than one possible pos: "..tostring(#possible_positions))
			local cur_distance = absdistance(path[#path].x,path[#path].y,x2,y2)
			print("Current distance: "..tostring(cur_distance))
			local flagged = {} -- a list of numbers to be removed from the table, highest to lowest
			for i = 1,#possible_positions do 
				local inspecting = #possible_positions - (i-1) 
				if absdistance(possible_positions[inspecting].x,possible_positions[inspecting].y,x2,y2) <= cur_distance then 
					print("Shorter choice found:"..possible_positions[inspecting].type)
				else
					table.insert(flagged,inspecting)
				end
			end
			for i = 1,#flagged do 
				print("Removing "..possible_positions[flagged[i]].type)
				table.remove(possible_positions,flagged[i])
			end
		end 
		--
		--check distance on each axis 
		if #possible_positions == 1 then 
			local c = {} 
			c.x = possible_positions[1].x
			c.y = possible_positions[1].y
			table.insert(path,c)
		else
			local x_distance = single_distance(path[#path].x,x2)
			local y_distance = single_distance(path[#path].y,y2)
			local choice = nil 
			if x_distance == y_distance then--if equal move on either axis 
				local d = math.random(1,2)
				if d == 1 then 
					choice = "x"
				else
					choice = "y"
				end
				print("New choice: "..choice)
			elseif x_distance > y_distance then -- move along the X axis 
				choice = "x"
			else -- move along the Y axis 
				choice = "y"
			end 
			print("choice:"..choice)
			--
			for i = 1,#possible_positions do 
				if choice == "x" then 
					if possible_positions[i].type == "left" or possible_positions.type == "right" then 
						local cur_distance = absdistance(path[#path].x,path[#path].y,x2,y2)
						if absdistance(possible_positions[i].x,possible_positions[i].y,x2,y2) <= cur_distance then 
							local c = {} 
							c.x = possible_positions[i].x
							c.y = possible_positions[i].y
							table.insert(path,c)
							break
						end
					end
				elseif choice == "y" then 
					if possible_positions[i].type == "up" or possible_positions.type == "down" then 
						local c = {} 
						c.x = possible_positions[i].x
						c.y = possible_positions[i].y
						table.insert(path,c)
						break
					end
				end
			end
		end
		if math.abs(absdistance(path[#path].x,path[#path].y,x2,y2)) <= 1 then 
			pathcomplete = true 
		end
	end
	current_path = path
end


function love.mousepressed(x,y,button)
	if button == 1 then path_timer = path_timermax end 
end
function love.update(dt)
	fps.update(dt)
	update_mouseover()
	if path_timer >= path_timermax then 
		update_path(24,24,mouseover.x,mouseover.y)
		path_timer = 0 
	end
end

function love.draw()
	--draw mouseover in green
	if mouseover ~= nil then 
		love.graphics.setColor(0,1,0)
		love.graphics.rectangle("fill",mouseover.x*15,mouseover.y*15,15,15)
	end
	--
	love.graphics.setColor(1,0,0)
	if #current_path >= 1 then 
		for i = 1,#current_path do 
			love.graphics.rectangle("fill",current_path[i].x*15,current_path[i].y*15,15,15)
		end
	end
end